#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2007-2008 Christian Dannie Storgaard
#
# AUTHOR:
# Christian Dannie Storgaard <cybolic@gmail.com>

import os, subprocess, sys, re

if 'WINEPREFIX' not in os.environ:
    os.environ['WINEPREFIX'] = os.path.expanduser("~/.wine")

import _registry
import _programs
import _version
import _bottles
import _drives
import _windows
import _audio
import _graphics
import _shellfolders
import _libraries
import desktop
import icons

_openargs = []

VERSION = subprocess.Popen(["wine", "--version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()[0].split("wine-")[1].strip()

for module in [_programs, _registry, _version, _bottles, _drives, _windows, _audio, _graphics, _shellfolders, _libraries]:
    if module.__name__ == 'wine._shellfolders':
        exec( '%s = module.ShellFolders()' % ( module.__name__.split('._')[1] ) )
    else:
        exec( '%s = module.%s()' % ( module.__name__.split('._')[1], module.__name__.split('._')[1].capitalize() ) )
    exec( '%s.VERSION = VERSION' % ( module.__name__.split('._')[1] ) )

def checkSetup():
    """
        Check whether the Wine installation has been created.
        Returns True or False
    """
    paths = [
        #The user profile directory isn't created by Wine until an application actually shoves something in there.
        #"%s/dosdevices/c:/windows/profiles/%s" % (self.env['WINEPREFIX'], self.env['USER']),
        "%s/dosdevices/c:/windows/system32" % os.environ['WINEPREFIX'],
        "%s/system.reg" % os.environ['WINEPREFIX'],
        "%s/userdef.reg" % os.environ['WINEPREFIX'],
        "%s/user.reg" % os.environ['WINEPREFIX'],
    ]
    return all([os.path.exists(path) for path in paths])

def getProgramName(exe):
    description = subprocess.Popen(["grep", "-ae", "<description>", exe], stdout=subprocess.PIPE, env=os.environ).communicate()[0]
    if description.strip():
        print description.split("<description>")[1].split("</description>")[0]
    else:
        print " ".join([ i.capitalize() for i in os.path.basename(exe)[:-4].split(" ") ])

def run(exe, use_url=True):
    """ Disable all warning and fixme messages from Wine's output """
    os.environ['WINEDEBUG'] = 'fixme-all'
    
    """ Remove any definition of the currect WINEPREFIX since this is already
        defined in the environment """
    exe = exe.replace('\\\\', '\\').replace('env WINEPREFIX="%s" ' % os.environ['WINEPREFIX'],'').replace('WINEPREFIX="%s" ' % os.environ['WINEPREFIX'],'')
    
    """ Get the extension of the last executable argument """
    ext = re.search(r'(?i).*\.(\w{3,4})', exe)
    if ext:
        ext = ext.groups()[0].lower()
    
    """ Remove wine command from the string and return the arguments """
    exe_args = util.splitString(exe.replace('wine ',''), remove_escapes=False)
    
    output_to_terminal = False
    executable = ['wine']
    
    
    if exe.strip() == 'regedit':
        output_to_terminal = True
        arguments = desktop._openargs+['regedit']
    elif exe.strip() == 'cmd':
        arguments = desktop._openargs+['wineconsole', '--backend=user', 'cmd']
    elif use_url and ext == 'url':
        url = re.search(r'(?i).*(\w\:\\.*?.url)["\']?$', exe)
        if url:
            urlfilename = util.wintounix(url.groups()[0])
            urlfile = open(urlfilename, 'r')
            url = urlfile.readlines()
            urlfile.close()
            url = re.search(r'(?im)\[InternetShortcut\]\s+URL=(.*?)\s+?$', '\n'.join(url))
            if url:
                url = url.groups()[0]
                output_to_terminal = True
                executable = ['xdg-open']
                arguments = [url]
        if not url:
            return run(exe, use_url=False)
    elif exe_args[0].lower().endswith('windows\\command\\start.exe') and os.path.isdir(util.wintounix(exe_args[1])):
        path = util.wintounix(exe_args[1])
        output_to_terminal = True
        executable = ['xdg-open']
        arguments = [path]
    elif exe_args[0].lower().endswith('windows\\system32\\cmd.exe'):
        arguments = desktop._openargs+['wineconsole', '--backend=user', 'cmd']+exe_args[1:]
    elif exe.startswith('env') or exe.startswith('wine '):
        arguments = util.splitString(exe.replace('wine ', ''), retain_defines=True)
    elif ext == 'msi':
        arguments = desktop._openargs+['msiexec', '/i']+[exe]
    else:
        arguments = desktop._openargs+[exe]
    
    if output_to_terminal:
        stdout = sys.stdout
        stderr = sys.stderr
    else:
        stdout = stderr = subprocess.PIPE
    
    try:
        process = subprocess.Popen(executable+arguments, stdout=stdout, stderr=stderr, env=os.environ)
        return process
    except OSError:
        if len(executable):
            print "Couldn't run command: %s" % executable+arguments
        else:
            print "Couldn't run command: %s" % arguments
        return False
